'use strict';

var escapeRegexp = require('escape-regex');

function createKeyRegex(str) {
  str = str.split('*').map(function (s) {
    return escapeRegexp(s);
  }).join('*');
  return new RegExp('^' + str.replace('*', '.*'));
}

function createPathFilter(options) {
  return function (key) {
    return options.ignoreWildcard ? key.startsWith(options.pathname) : key.match(createKeyRegex(options.pathname));
  };
}

function filterPaths(paths, options) {
  return (paths || []).filter(createPathFilter(options));
}

function createViewFilter(options) {
  return function (view) {
    if (options && options.before && view.time > options.before) return false;
    if (options && options.after && view.time < options.after) return false;
    return true;
  };
}

function filterViews(views, options) {
  return (views || []).filter(createViewFilter(options));
}

module.exports = {
  createKeyRegex: createKeyRegex,
  createPathFilter: createPathFilter,
  filterPaths: filterPaths,
  createViewFilter: createViewFilter,
  filterViews: filterViews
};