'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

module.exports = function testAdapter(options) {
  var _this = this;

  describe('micro-analytics adapter ' + options.name, function () {
    var adapter = require(options.modulePath);

    if (typeof options.beforeEach === 'function') {
      beforeEach(_asyncToGenerator(_regenerator2.default.mark(function _callee() {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return options.beforeEach(adapter);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this);
      })));
    }

    if (typeof options.afterEach === 'function') {
      afterEach(_asyncToGenerator(_regenerator2.default.mark(function _callee2() {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return options.afterEach(adapter);

              case 2:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, _this);
      })));
    }

    if (typeof options.beforeAll === 'function') {
      beforeAll(_asyncToGenerator(_regenerator2.default.mark(function _callee3() {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return options.beforeAll(adapter);

              case 2:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this);
      })));
    }

    if (typeof options.afterAll === 'function') {
      afterAll(_asyncToGenerator(_regenerator2.default.mark(function _callee4() {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return options.afterAll(adapter);

              case 2:
                if (!(typeof adapter.close === 'function')) {
                  _context4.next = 5;
                  break;
                }

                _context4.next = 5;
                return adapter.close();

              case 5:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, _this);
      })));
    }

    test('get() should return a promise', function () {
      expect(adapter.get('/a-key').constructor.name).toEqual('Promise');
    });

    test('getAll() should return a promise', function () {
      expect(adapter.getAll({ pathname: '/' }).constructor.name).toEqual('Promise');
    });

    test('has() should return a promise', function () {
      expect(adapter.has('/a-key').constructor.name).toEqual('Promise');
    });

    test('keys() should return a promise', function () {
      expect(adapter.keys().constructor.name).toEqual('Promise');
    });

    test('put() should return a promise', function () {
      expect(adapter.put('/a-key', {}).constructor.name).toEqual('Promise');
    });

    if (typeof adapter.options !== 'undefined') {
      test('options should be an array of args options', function () {
        expect(Array.isArray(adapter.options)).toBe(true);
        if (adapter.options.length >= 0) {
          var counter = 0;
          adapter.options.forEach(function (option) {
            expect(option.name).toBeDefined();
            expect(option.description).toBeDefined();
            counter++;
          });

          // if the forEach somehow breaks the test should break
          expect(counter).toBe(adapter.options.length);
        }
      });

      test('init should be a defined when options is defined', function () {
        expect(_typeof(adapter.init)).not.toBe('undefined');
      });
    } else {
      test.skip('options should be an array of args options', function () {});
      test.skip('init should be a defined when options is defined', function () {});
    }

    if (typeof adapter.init !== 'undefined') {
      test('init should be a function', function () {
        expect(_typeof(adapter.init)).toBe('function');
      });

      test('call init should not throw', function () {
        adapter.init(options.initOptions || {});
      });
    } else {
      test.skip('init should be a function', function () {});
      test.skip('call init should not throw', function () {});
    }

    it('should save and read', _asyncToGenerator(_regenerator2.default.mark(function _callee5() {
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return adapter.put('/a-key', { views: [{ time: 1490623474639 }] });

            case 2:
              _context5.t0 = expect;
              _context5.next = 5;
              return adapter.get('/a-key');

            case 5:
              _context5.t1 = _context5.sent;
              _context5.t2 = {
                views: [{ time: 1490623474639 }]
              };
              (0, _context5.t0)(_context5.t1).toEqual(_context5.t2);

            case 8:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, _this);
    })));

    it('should return empty list of views when key has no views', _asyncToGenerator(_regenerator2.default.mark(function _callee6() {
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              _context6.t0 = expect;
              _context6.next = 3;
              return adapter.get('/c-key');

            case 3:
              _context6.t1 = _context6.sent;
              _context6.t2 = { views: [] };
              (0, _context6.t0)(_context6.t1).toEqual(_context6.t2);

            case 6:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, _this);
    })));

    it('should return all saves on getAll', _asyncToGenerator(_regenerator2.default.mark(function _callee7() {
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              _context7.next = 2;
              return adapter.put('/a-key', { views: [{ time: 1490623474639 }] });

            case 2:
              _context7.next = 4;
              return adapter.put('/another-key', { views: [{ time: 1490623474639 }] });

            case 4:
              _context7.t0 = expect;
              _context7.next = 7;
              return adapter.getAll({ pathname: '/' });

            case 7:
              _context7.t1 = _context7.sent;
              _context7.t2 = {
                '/a-key': { views: [{ time: 1490623474639 }] },
                '/another-key': { views: [{ time: 1490623474639 }] }
              };
              (0, _context7.t0)(_context7.t1).toEqual(_context7.t2);

            case 10:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, _this);
    })));

    it('should return filtered saves from getAll based on pathname', _asyncToGenerator(_regenerator2.default.mark(function _callee8() {
      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              _context8.next = 2;
              return adapter.put('/a-key', { views: [{ time: 1490623474639 }] });

            case 2:
              _context8.next = 4;
              return adapter.put('/another-key', { views: [{ time: 1490623474639 }] });

            case 4:
              _context8.next = 6;
              return adapter.put('/b-key', { views: [{ time: 1490623474639 }] });

            case 6:
              _context8.t0 = expect;
              _context8.next = 9;
              return adapter.getAll({ pathname: '/a' });

            case 9:
              _context8.t1 = _context8.sent;
              _context8.t2 = {
                '/a-key': { views: [{ time: 1490623474639 }] },
                '/another-key': { views: [{ time: 1490623474639 }] }
              };
              (0, _context8.t0)(_context8.t1).toEqual(_context8.t2);

            case 12:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, _this);
    })));

    it('should return filtered saves from getAll based on before', _asyncToGenerator(_regenerator2.default.mark(function _callee9() {
      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              _context9.next = 2;
              return adapter.put('/a-key', { views: [{ time: 1490623474639 }] });

            case 2:
              _context9.next = 4;
              return adapter.put('/another-key', { views: [{ time: 1490623478639 }] });

            case 4:
              _context9.next = 6;
              return adapter.put('/b-key', { views: [{ time: 1490623484639 }] });

            case 6:
              _context9.t0 = expect;
              _context9.next = 9;
              return adapter.getAll({ pathname: '/', before: 1490623478640 });

            case 9:
              _context9.t1 = _context9.sent;
              _context9.t2 = {
                '/a-key': { views: [{ time: 1490623474639 }] },
                '/another-key': { views: [{ time: 1490623478639 }] },
                '/b-key': { views: [] }
              };
              (0, _context9.t0)(_context9.t1).toEqual(_context9.t2);

            case 12:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, _this);
    })));

    it('should return filtered saves from getAll based on after', _asyncToGenerator(_regenerator2.default.mark(function _callee10() {
      return _regenerator2.default.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _context10.next = 2;
              return adapter.put('/a-key', { views: [{ time: 1490623474639 }] });

            case 2:
              _context10.next = 4;
              return adapter.put('/another-key', { views: [{ time: 1490623478639 }] });

            case 4:
              _context10.next = 6;
              return adapter.put('/b-key', { views: [{ time: 1490623484639 }] });

            case 6:
              _context10.t0 = expect;
              _context10.next = 9;
              return adapter.getAll({ pathname: '/', after: 1490623478638 });

            case 9:
              _context10.t1 = _context10.sent;
              _context10.t2 = {
                '/a-key': { views: [] },
                '/another-key': { views: [{ time: 1490623478639 }] },
                '/b-key': { views: [{ time: 1490623484639 }] }
              };
              (0, _context10.t0)(_context10.t1).toEqual(_context10.t2);

            case 12:
            case 'end':
              return _context10.stop();
          }
        }
      }, _callee10, _this);
    })));

    it('should return filtered saves from get based on before', _asyncToGenerator(_regenerator2.default.mark(function _callee11() {
      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              _context11.next = 2;
              return adapter.put('/a-key', {
                views: [{ time: 1490623474639 }, { time: 1490623478639 }]
              });

            case 2:
              _context11.t0 = expect;
              _context11.next = 5;
              return adapter.get('/a-key', { before: 1490623475640 });

            case 5:
              _context11.t1 = _context11.sent;
              _context11.t2 = {
                views: [{ time: 1490623474639 }]
              };
              (0, _context11.t0)(_context11.t1).toEqual(_context11.t2);

            case 8:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, _this);
    })));

    it('should return filtered saves from get based on after', _asyncToGenerator(_regenerator2.default.mark(function _callee12() {
      return _regenerator2.default.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              _context12.next = 2;
              return adapter.put('/a-key', {
                views: [{ time: 1490623474639 }, { time: 1490623478639 }]
              });

            case 2:
              _context12.t0 = expect;
              _context12.next = 5;
              return adapter.get('/a-key', { after: 1490623475640 });

            case 5:
              _context12.t1 = _context12.sent;
              _context12.t2 = {
                views: [{ time: 1490623478639 }]
              };
              (0, _context12.t0)(_context12.t1).toEqual(_context12.t2);

            case 8:
            case 'end':
              return _context12.stop();
          }
        }
      }, _callee12, _this);
    })));

    it('should have check whether a key is stored with has', _asyncToGenerator(_regenerator2.default.mark(function _callee13() {
      return _regenerator2.default.wrap(function _callee13$(_context13) {
        while (1) {
          switch (_context13.prev = _context13.next) {
            case 0:
              _context13.next = 2;
              return adapter.put('/a-key', { views: [{ time: 1490623474639 }] });

            case 2:
              _context13.t0 = expect;
              _context13.next = 5;
              return adapter.has('/a-key');

            case 5:
              _context13.t1 = _context13.sent;
              (0, _context13.t0)(_context13.t1).toEqual(true);
              _context13.t2 = expect;
              _context13.next = 10;
              return adapter.has('/non-existing-key');

            case 10:
              _context13.t3 = _context13.sent;
              (0, _context13.t2)(_context13.t3).toEqual(false);

            case 12:
            case 'end':
              return _context13.stop();
          }
        }
      }, _callee13, _this);
    })));

    if (typeof adapter.subscribe === 'function') {
      it('should allow subscription with observables', _asyncToGenerator(_regenerator2.default.mark(function _callee14() {
        var listener, unsubscribe;
        return _regenerator2.default.wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                listener = jest.fn();
                unsubscribe = adapter.subscribe(listener);
                _context14.next = 4;
                return adapter.put('/a-key', { views: [{ time: 1490623474639 }] });

              case 4:

                expect(listener).toHaveBeenCalledWith({
                  key: '/a-key',
                  value: { views: [{ time: 1490623474639 }] }
                });

              case 5:
              case 'end':
                return _context14.stop();
            }
          }
        }, _callee14, _this);
      })));

      it('should allow multiple subscription with observables and handle unsubscribption', _asyncToGenerator(_regenerator2.default.mark(function _callee15() {
        var listener1, listener2, subscription;
        return _regenerator2.default.wrap(function _callee15$(_context15) {
          while (1) {
            switch (_context15.prev = _context15.next) {
              case 0:
                listener1 = jest.fn();
                listener2 = jest.fn();
                subscription = adapter.subscribe(listener1);

                adapter.subscribe(listener2);

                _context15.next = 6;
                return adapter.put('/a-key', { views: [{ time: 1490623474639 }] });

              case 6:
                subscription.unsubscribe();
                _context15.next = 9;
                return adapter.put('/b-key', { views: [{ time: 1490623474639 }] });

              case 9:

                expect(listener1).toHaveBeenCalledWith({
                  key: '/a-key',
                  value: { views: [{ time: 1490623474639 }] }
                });
                expect(listener1).not.toHaveBeenCalledWith({
                  key: '/b-key',
                  value: { views: [{ time: 1490623474639 }] }
                });
                expect(listener2).toHaveBeenCalledWith({
                  key: '/a-key',
                  value: { views: [{ time: 1490623474639 }] }
                });
                expect(listener2).toHaveBeenCalledWith({
                  key: '/b-key',
                  value: { views: [{ time: 1490623474639 }] }
                });

              case 13:
              case 'end':
                return _context15.stop();
            }
          }
        }, _callee15, _this);
      })));
    } else {
      it.skip('should allow subscription with observables', function () {});
      it.skip('should allow multiple subscription with observables and handle unsubscribption', function () {});
    }
  });
};